program worker

use mpi

implicit none

integer rank, sz, ierr

integer i, stat(MPI_STATUS_SIZE)
integer, parameter :: dummy_tag = 42
integer, parameter :: nfac = 20 ! compute faculties up to nfac
integer :: n, w ! next work item, worker rank
real :: result  ! single result
real, dimension(nfac) :: results
integer, allocatable, dimension(:) :: assigned

call MPI_INIT(ierr)
call MPI_COMM_RANK(MPI_COMM_WORLD, rank, ierr)
call MPI_COMM_SIZE(MPI_COMM_WORLD, sz, ierr)

if (rank .eq. 0) then
  allocate(assigned(sz - 1))
  ! send some intial work to all workers
  n = 1 ! work item
  do i=1,sz-1
    assigned(i) = n ! record n assigned to worker w
    call MPI_Send(assigned(i), 1, MPI_INTEGER, i, dummy_tag, MPI_COMM_WORLD,&
                  ierr)
    n = n + 1
  end do
  ! send out remaining work until done whenever we receive a result
  do i=1,nfac
    ! wait for any worker to return result
    call MPI_Recv(result, 1, MPI_REAL, MPI_ANY_SOURCE, MPI_ANY_TAG,&
                  MPI_COMM_WORLD, stat, ierr)
    w = stat(MPI_SOURCE) ! find out rank of sender
    results(assigned(w)) = result
    print *,'Received ',assigned(w),'! = ',result,' from worker ',w
    ! hand out next work item
    assigned(w) = n ! record n assigned to worker w
    call MPI_Send(assigned(w), 1, MPI_INTEGER, w, dummy_tag, MPI_COMM_WORLD,&
                  ierr)
    n = n + 1
  end do
  ! tell all workers to quit by sending magic work item
  do i=1,sz-1
    n = 0
    call MPI_Send(n, 1, MPI_INTEGER, i, dummy_tag, MPI_COMM_WORLD,ierr)
  end do

  ! output result
  do i=1,nfac
    print *,i,'! = ',results(i)
  end do
else
  do while(.True.)
    ! wait for work assignment from manager
    call MPI_Recv(n, 1, MPI_INTEGER, 0, dummy_tag, MPI_COMM_WORLD, stat, ierr)
    if(n .eq. 0) then 
      exit
    endif
    ! compute faculty of given number, store in result
    result = 1.
    do i=1,n
     result = result * i
    end do
    call sleep(rank) ! pretent that computation took some time
    call MPI_Send(result, 1, MPI_REAL, 0, dummy_tag, MPI_COMM_WORLD, ierr)
  end do
end if


call MPI_FINALIZE(ierr)

end program worker
